import math
import sys
import random
from typing import List

def radixsort2(lst: List[int], P: List[int]) -> List[int]:
    n = len(lst)
    k = math.floor(math.log(max(P), 2))

    if not args.silent:
        print(f'Instructions: Take the {n} objects and put them in one stack. Make space for two more stacks. To execute one row of numbers, take as many objects from the top of the first stack and place them onto the two other stacks, alternating between them. For example, to execute the row "1 2 3 2 1 2", you deal the top object of the first stack onto the second stack, the next one onto the third stack, the one after that again on the third stack, the three objects after that onto the second one, and so on. Take care to deal one object at a time. To execute a transfer, move the second stack into the place of the first stack, and the third stack on top of it.\n')

    if not k&1: P = [n-1-i for i in P]
    for bit in range(k+1):
        lst0 = []
        lst1 = []
        moves = []
        for i in lst:
            val = ((P[i] >> bit) ^ bit) & 1
            moves.append(val)
            if val: lst1.append(i)
            else:   lst0.append(i)
        lst = lst0 + lst1
        lst.reverse()

        last = 0
        count = 0
        numbers = []
        for i in moves:
            if i == last: count += 1
            else:
                numbers.append(count)
                last = i
                count = 1
        if count: numbers.append(count)

        if bit: print()
        for it in range(0, len(numbers), 6):
            print('  ' + ' '.join(map(str, numbers[it:it+6])))
        print('transfer')

    if args.result:
        print('\nResulting order, where 1 is the top element of the original stack:')
        for it in range(0, len(lst), 6):
            print('  ' + ', '.join(map(str, lst[it:it+6])))
    return lst

import argparse
parser = argparse.ArgumentParser(description='Automatically generate instructions to shuffle a stack of objects.')
parser.add_argument('n', type=int, help='The number of objects.')
parser.add_argument('-s', '--silent', action='store_true', help='Do not print instructions.')
parser.add_argument('-r', '--result', action='store_true', help='Print the resulting ordering.')

args = parser.parse_args()

lst = list(range(args.n))
P   = list(range(args.n))
random.shuffle(P)
lst = radixsort2(lst, P)
assert all(it == P[i] for it, i in enumerate(lst))
